/*
   Copyright (c) 2013 deeptought 

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 

   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 

   The Software shall be used for Good, not Evil. 

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */

package de.uks.se.castle;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;

import de.uks.se.castle.creators.CastlePO;
import de.uks.se.castle.creators.ModelPattern;
import de.uks.se.castle.creators.PersonPO;
import de.uks.se.castle.creators.PrincessPO;
import de.uks.se.castle.creators.RoomPO;
import de.uks.se.castle.creators.RoomSet;
import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;
import de.uks.se.castle.creators.PersonSet;

public class Castle implements PropertyChangeInterface
{


	//==========================================================================

	public Object get(String attrName)
	{
		if (PROPERTY_ROOMS.equalsIgnoreCase(attrName))
		{
			return getRooms();
		}

		if (PROPERTY_PERSONS.equalsIgnoreCase(attrName))
		{
			return getPersons();
		}

		return null;
	}


	//==========================================================================

	public boolean set(String attrName, Object value)
	{
		if (PROPERTY_ROOMS.equalsIgnoreCase(attrName))
		{
			addToRooms((Room) value);
			return true;
		}

		if ((PROPERTY_ROOMS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
		{
			removeFromRooms((Room) value);
			return true;
		}

		if (PROPERTY_PERSONS.equalsIgnoreCase(attrName))
		{
			addToPersons((Person) value);
			return true;
		}

		if ((PROPERTY_PERSONS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
		{
			removeFromPersons((Person) value);
			return true;
		}

		return false;
	}


	//==========================================================================

	protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	public PropertyChangeSupport getPropertyChangeSupport()
	{
		return listeners;
	}


	//==========================================================================

	public void removeYou()
	{
		removeAllFromRooms();
		removeAllFromPersons();
		getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
	}


	//==========================================================================

	public int countAllItems(  )
	{
		int result = 0;

		ModelPattern countPattern = new ModelPattern();
		CastlePO castlePO = countPattern.hasElementCastlePO(this);
		RoomPO roomPO = castlePO.hasRooms();

		while(countPattern.getHasMatch())
		{
			result += roomPO.getCurrentMatch().getItems().size();

			countPattern.findNextMatch();
		}

		return result;
	}

	// ALTERNATIVE
	// 	public int countAllItems(  )
	// 	{
	// 		int result = 0;
	// 		
	// 		ModelPattern countPattern = new ModelPattern();
	// 		CastlePO castlePO = countPattern.hasElementCastlePO(this);
	// 		RoomPO roomPO = castlePO.hasRooms();
	// 		ItemPO itemPO = roomPO.hasItems();
	// 		
	// 		while(countPattern.getHasMatch())
	// 		{
	// 			result++;
	// 			
	// 			countPattern.findNextMatch();
	// 		}
	//
	// 		return result;
	// 	}


	//==========================================================================

	public void princessSaved(  )
	{
		ModelPattern princessPattern = new ModelPattern();

		//matching
		CastlePO castlePO = princessPattern.hasElementCastlePO(this);
		RoomPO roomPO = castlePO.hasRooms();
		PersonPO princePO = roomPO.hasPersons();
		PrincessPO princessPO = roomPO.hasPrincess();

		princessPattern.startCreate();

		//create Link between prince and princess
		princePO.hasSavedPrincess(princessPO);
	}


	/********************************************************************
	 * <pre>
	 *              one                       many
	 * Castle ----------------------------------- Room
	 *              castle                   rooms
	 * </pre>
	 */

	public static final String PROPERTY_ROOMS = "rooms";

	private RoomSet rooms = null;

	public RoomSet getRooms()
	{
		if (this.rooms == null)
		{
			return Room.EMPTY_SET;
		}

		return this.rooms;
	}

	public boolean addToRooms(Room value)
	{
		boolean changed = false;

		if (value != null)
		{
			if (this.rooms == null)
			{
				this.rooms = new RoomSet();
			}

			changed = this.rooms.add (value);

			if (changed)
			{
				value.withCastle(this);
				getPropertyChangeSupport().firePropertyChange(PROPERTY_ROOMS, null, value);
			}
		}

		return changed;   
	}

	public boolean removeFromRooms(Room value)
	{
		boolean changed = false;

		if ((this.rooms != null) && (value != null))
		{
			changed = this.rooms.remove (value);

			if (changed)
			{
				value.setCastle(null);
				getPropertyChangeSupport().firePropertyChange(PROPERTY_ROOMS, value, null);
			}
		}

		return changed;   
	}

	public Castle withRooms(Room value)
	{
		addToRooms(value);
		return this;
	} 

	public Castle withoutRooms(Room value)
	{
		removeFromRooms(value);
		return this;
	} 

	public void removeAllFromRooms()
	{
		LinkedHashSet<Room> tmpSet = new LinkedHashSet<Room>(this.getRooms());

		for (Room value : tmpSet)
		{
			this.removeFromRooms(value);
		}
	}

	public Room createRooms()
	{
		Room value = new Room();
		withRooms(value);
		return value;
	} 


	/********************************************************************
	 * <pre>
	 *              one                       many
	 * Castle ----------------------------------- Person
	 *              castle                   persons
	 * </pre>
	 */

	 public static final String PROPERTY_PERSONS = "persons";

	 private PersonSet persons = null;

	 public PersonSet getPersons()
	 {
		 if (this.persons == null)
		 {
			 return Person.EMPTY_SET;
		 }

		 return this.persons;
	 }

	 public boolean addToPersons(Person value)
	 {
		 boolean changed = false;

		 if (value != null)
		 {
			 if (this.persons == null)
			 {
				 this.persons = new PersonSet();
			 }

			 changed = this.persons.add (value);

			 if (changed)
			 {
				 value.withCastle(this);
				 getPropertyChangeSupport().firePropertyChange(PROPERTY_PERSONS, null, value);
			 }
		 }

		 return changed;   
	 }

	 public boolean removeFromPersons(Person value)
	 {
		 boolean changed = false;

		 if ((this.persons != null) && (value != null))
		 {
			 changed = this.persons.remove (value);

			 if (changed)
			 {
				 value.setCastle(null);
				 getPropertyChangeSupport().firePropertyChange(PROPERTY_PERSONS, value, null);
			 }
		 }

		 return changed;   
	 }

	 public Castle withPersons(Person value)
	 {
		 addToPersons(value);
		 return this;
	 } 

	 public Castle withoutPersons(Person value)
	 {
		 removeFromPersons(value);
		 return this;
	 } 

	 public void removeAllFromPersons()
	 {
		 LinkedHashSet<Person> tmpSet = new LinkedHashSet<Person>(this.getPersons());

		 for (Person value : tmpSet)
		 {
			 this.removeFromPersons(value);
		 }
	 }

	 public Person createPersons()
	 {
		 Person value = new Person();
		 withPersons(value);
		 return value;
	 } 
}

