/*
   Copyright (c) 2013 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uks.se.castle;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import org.sdmlib.utils.StrUtil;
import de.uks.se.castle.creators.ItemSet;

public class Item implements PropertyChangeInterface
{

   
   //==========================================================================
   
   public Object get(String attrName)
   {
      if (PROPERTY_DESCRIPTION.equalsIgnoreCase(attrName))
      {
         return getDescription();
      }

      if (PROPERTY_QUALITY.equalsIgnoreCase(attrName))
      {
         return getQuality();
      }

      if (PROPERTY_ROOM.equalsIgnoreCase(attrName))
      {
         return getRoom();
      }

      if (PROPERTY_PERSON.equalsIgnoreCase(attrName))
      {
         return getPerson();
      }

      return null;
   }

   
   //==========================================================================
   
   public boolean set(String attrName, Object value)
   {
      if (PROPERTY_DESCRIPTION.equalsIgnoreCase(attrName))
      {
         setDescription((String) value);
         return true;
      }

      if (PROPERTY_QUALITY.equalsIgnoreCase(attrName))
      {
         setQuality(Integer.parseInt(value.toString()));
         return true;
      }

      if (PROPERTY_ROOM.equalsIgnoreCase(attrName))
      {
         setRoom((Room) value);
         return true;
      }

      if (PROPERTY_PERSON.equalsIgnoreCase(attrName))
      {
         setPerson((Person) value);
         return true;
      }

      return false;
   }

   
   //==========================================================================
   
   protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);
   
   public PropertyChangeSupport getPropertyChangeSupport()
   {
      return listeners;
   }

   
   //==========================================================================
   
   public void removeYou()
   {
      setRoom(null);
      setPerson(null);
      getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
   }

   
   //==========================================================================
   
   public static final String PROPERTY_DESCRIPTION = "description";
   
   private String description;

   public String getDescription()
   {
      return this.description;
   }
   
   public void setDescription(String value)
   {
      if ( ! StrUtil.stringEquals(this.description, value))
      {
         String oldValue = this.description;
         this.description = value;
         getPropertyChangeSupport().firePropertyChange(PROPERTY_DESCRIPTION, oldValue, value);
      }
   }
   
   public Item withDescription(String value)
   {
      setDescription(value);
      return this;
   } 

   public String toString()
   {
      StringBuilder _ = new StringBuilder();
      
      _.append(" ").append(this.getDescription());
      _.append(" ").append(this.getQuality());
      return _.substring(1);
   }


   
   //==========================================================================
   
   public static final String PROPERTY_QUALITY = "quality";
   
   private int quality;

   public int getQuality()
   {
      return this.quality;
   }
   
   public void setQuality(int value)
   {
      if (this.quality != value)
      {
         int oldValue = this.quality;
         this.quality = value;
         getPropertyChangeSupport().firePropertyChange(PROPERTY_QUALITY, oldValue, value);
      }
   }
   
   public Item withQuality(int value)
   {
      setQuality(value);
      return this;
   } 

   
   public static final ItemSet EMPTY_SET = new ItemSet();

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Item ----------------------------------- Room
    *              items                   room
    * </pre>
    */
   
   public static final String PROPERTY_ROOM = "room";
   
   private Room room = null;
   
   public Room getRoom()
   {
      return this.room;
   }
   
   public boolean setRoom(Room value)
   {
      boolean changed = false;
      
      if (this.room != value)
      {
         Room oldValue = this.room;
         
         if (this.room != null)
         {
            this.room = null;
            oldValue.withoutItems(this);
         }
         
         this.room = value;
         
         if (value != null)
         {
            value.withItems(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_ROOM, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Item withRoom(Room value)
   {
      setRoom(value);
      return this;
   } 
   
   public Room createRoom()
   {
      Room value = new Room();
      withRoom(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Item ----------------------------------- Person
    *              items                   person
    * </pre>
    */
   
   public static final String PROPERTY_PERSON = "person";
   
   private Person person = null;
   
   public Person getPerson()
   {
      return this.person;
   }
   
   public boolean setPerson(Person value)
   {
      boolean changed = false;
      
      if (this.person != value)
      {
         Person oldValue = this.person;
         
         if (this.person != null)
         {
            this.person = null;
            oldValue.withoutItems(this);
         }
         
         this.person = value;
         
         if (value != null)
         {
            value.withItems(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_PERSON, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Item withPerson(Person value)
   {
      setPerson(value);
      return this;
   } 
   
   public Person createPerson()
   {
      Person value = new Person();
      withPerson(value);
      return value;
   } 
}

