/*
   Copyright (c) 2013 deeptought 

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 

   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 

   The Software shall be used for Good, not Evil. 

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */

package de.uks.se.castle;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import org.sdmlib.utils.StrUtil;

import de.uks.se.castle.creators.ItemPO;
import de.uks.se.castle.creators.ModelPattern;
import de.uks.se.castle.creators.PersonPO;
import de.uks.se.castle.creators.PersonSet;
import de.uks.se.castle.creators.ItemSet;
import de.uks.se.castle.creators.RoomPO;

import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;

public class Person implements PropertyChangeInterface
{


	//==========================================================================

	public Object get(String attrName)
	{
		if (PROPERTY_NAME.equalsIgnoreCase(attrName))
		{
			return getName();
		}

		if (PROPERTY_EXPERIENCE.equalsIgnoreCase(attrName))
		{
			return getExperience();
		}

		if (PROPERTY_CASTLE.equalsIgnoreCase(attrName))
		{
			return getCastle();
		}

		if (PROPERTY_ROOM.equalsIgnoreCase(attrName))
		{
			return getRoom();
		}

		if (PROPERTY_ITEMS.equalsIgnoreCase(attrName))
		{
			return getItems();
		}

		if (PROPERTY_SAVEDPRINCESS.equalsIgnoreCase(attrName))
		{
			return getSavedPrincess();
		}

		return null;
	}


	//==========================================================================

	public boolean set(String attrName, Object value)
	{
		if (PROPERTY_NAME.equalsIgnoreCase(attrName))
		{
			setName((String) value);
			return true;
		}

		if (PROPERTY_EXPERIENCE.equalsIgnoreCase(attrName))
		{
			setExperience(Integer.parseInt(value.toString()));
			return true;
		}

		if (PROPERTY_CASTLE.equalsIgnoreCase(attrName))
		{
			setCastle((Castle) value);
			return true;
		}

		if (PROPERTY_ROOM.equalsIgnoreCase(attrName))
		{
			setRoom((Room) value);
			return true;
		}

		if (PROPERTY_ITEMS.equalsIgnoreCase(attrName))
		{
			addToItems((Item) value);
			return true;
		}

		if ((PROPERTY_ITEMS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
		{
			removeFromItems((Item) value);
			return true;
		}

		if (PROPERTY_SAVEDPRINCESS.equalsIgnoreCase(attrName))
		{
			setSavedPrincess((Princess) value);
			return true;
		}

		return false;
	}


	//==========================================================================

	protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	public PropertyChangeSupport getPropertyChangeSupport()
	{
		return listeners;
	}


	//==========================================================================

	public void removeYou()
	{
		setCastle(null);
		setRoom(null);
		removeAllFromItems();
		setSavedPrincess(null);
		getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
	}


	//==========================================================================

	public void collectItemsInCurrentRoom(  )
	{
		ModelPattern collectItemsPattern = new ModelPattern();

		PersonPO personPO = collectItemsPattern.hasElementPersonPO(this);
		RoomPO currentRoomPO = personPO.hasRoom();
		ItemPO itemPO = currentRoomPO.hasItems();

		while (collectItemsPattern.getHasMatch()) {
			Person person = personPO.getCurrentMatch();

			person.addToItems(itemPO.getCurrentMatch());

			int newExperience = person.getExperience() + itemPO.getCurrentMatch().getQuality();
			person.setExperience(newExperience);

			collectItemsPattern.findNextMatch();
		}
	}


	//==========================================================================

	public static final String PROPERTY_NAME = "name";

	private String name;

	public String getName()
	{
		return this.name;
	}

	public void setName(String value)
	{
		if ( ! StrUtil.stringEquals(this.name, value))
		{
			String oldValue = this.name;
			this.name = value;
			getPropertyChangeSupport().firePropertyChange(PROPERTY_NAME, oldValue, value);
		}
	}

	public Person withName(String value)
	{
		setName(value);
		return this;
	} 

	public String toString()
	{
		StringBuilder _ = new StringBuilder();

		_.append(" ").append(this.getName());
		_.append(" ").append(this.getExperience());
		return _.substring(1);
	}



	//==========================================================================

	public static final String PROPERTY_EXPERIENCE = "experience";

	private int experience;

	public int getExperience()
	{
		return this.experience;
	}

	public void setExperience(int value)
	{
		if (this.experience != value)
		{
			int oldValue = this.experience;
			this.experience = value;
			getPropertyChangeSupport().firePropertyChange(PROPERTY_EXPERIENCE, oldValue, value);
		}
	}

	public Person withExperience(int value)
	{
		setExperience(value);
		return this;
	} 


	public static final PersonSet EMPTY_SET = new PersonSet();


	/********************************************************************
	 * <pre>
	 *              many                       one
	 * Person ----------------------------------- Castle
	 *              persons                   castle
	 * </pre>
	 */

	public static final String PROPERTY_CASTLE = "castle";

	private Castle castle = null;

	public Castle getCastle()
	{
		return this.castle;
	}

	public boolean setCastle(Castle value)
	{
		boolean changed = false;

		if (this.castle != value)
		{
			Castle oldValue = this.castle;

			if (this.castle != null)
			{
				this.castle = null;
				oldValue.withoutPersons(this);
			}

			this.castle = value;

			if (value != null)
			{
				value.withPersons(this);
			}

			getPropertyChangeSupport().firePropertyChange(PROPERTY_CASTLE, oldValue, value);
			changed = true;
		}

		return changed;
	}

	public Person withCastle(Castle value)
	{
		setCastle(value);
		return this;
	} 

	public Castle createCastle()
	{
		Castle value = new Castle();
		withCastle(value);
		return value;
	} 


	/********************************************************************
	 * <pre>
	 *              many                       one
	 * Person ----------------------------------- Room
	 *              persons                   room
	 * </pre>
	 */

	public static final String PROPERTY_ROOM = "room";

	private Room room = null;

	public Room getRoom()
	{
		return this.room;
	}

	public boolean setRoom(Room value)
	{
		boolean changed = false;

		if (this.room != value)
		{
			Room oldValue = this.room;

			if (this.room != null)
			{
				this.room = null;
				oldValue.withoutPersons(this);
			}

			this.room = value;

			if (value != null)
			{
				value.withPersons(this);
			}

			getPropertyChangeSupport().firePropertyChange(PROPERTY_ROOM, oldValue, value);
			changed = true;
		}

		return changed;
	}

	public Person withRoom(Room value)
	{
		setRoom(value);
		return this;
	} 

	public Room createRoom()
	{
		Room value = new Room();
		withRoom(value);
		return value;
	} 


	/********************************************************************
	 * <pre>
	 *              one                       many
	 * Person ----------------------------------- Item
	 *              person                   items
	 * </pre>
	 */

	public static final String PROPERTY_ITEMS = "items";

	private ItemSet items = null;

	public ItemSet getItems()
	{
		if (this.items == null)
		{
			return Item.EMPTY_SET;
		}

		return this.items;
	}

	public boolean addToItems(Item value)
	{
		boolean changed = false;

		if (value != null)
		{
			if (this.items == null)
			{
				this.items = new ItemSet();
			}

			changed = this.items.add (value);

			if (changed)
			{
				value.withPerson(this);
				getPropertyChangeSupport().firePropertyChange(PROPERTY_ITEMS, null, value);
			}
		}

		return changed;   
	}

	public boolean removeFromItems(Item value)
	{
		boolean changed = false;

		if ((this.items != null) && (value != null))
		{
			changed = this.items.remove (value);

			if (changed)
			{
				value.setPerson(null);
				getPropertyChangeSupport().firePropertyChange(PROPERTY_ITEMS, value, null);
			}
		}

		return changed;   
	}

	public Person withItems(Item value)
	{
		addToItems(value);
		return this;
	} 

	public Person withoutItems(Item value)
	{
		removeFromItems(value);
		return this;
	} 

	public void removeAllFromItems()
	{
		LinkedHashSet<Item> tmpSet = new LinkedHashSet<Item>(this.getItems());

		for (Item value : tmpSet)
		{
			this.removeFromItems(value);
		}
	}

	public Item createItems()
	{
		Item value = new Item();
		withItems(value);
		return value;
	} 


	/********************************************************************
	 * <pre>
	 *              one                       one
	 * Person ----------------------------------- Princess
	 *              prince                   savedPrincess
	 * </pre>
	 */

	public static final String PROPERTY_SAVEDPRINCESS = "savedPrincess";

	private Princess savedPrincess = null;

	public Princess getSavedPrincess()
	{
		return this.savedPrincess;
	}

	public boolean setSavedPrincess(Princess value)
	{
		boolean changed = false;

		if (this.savedPrincess != value)
		{
			Princess oldValue = this.savedPrincess;

			if (this.savedPrincess != null)
			{
				this.savedPrincess = null;
				oldValue.setPrince(null);
			}

			this.savedPrincess = value;

			if (value != null)
			{
				value.withPrince(this);
			}

			getPropertyChangeSupport().firePropertyChange(PROPERTY_SAVEDPRINCESS, oldValue, value);
			changed = true;
		}

		return changed;
	}

	public Person withSavedPrincess(Princess value)
	{
		setSavedPrincess(value);
		return this;
	} 

	public Princess createSavedPrincess()
	{
		Princess value = new Princess();
		withSavedPrincess(value);
		return value;
	} 
}

