/*
   Copyright (c) 2013 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uks.se.castle;

import org.sdmlib.utils.PropertyChangeInterface;
import java.beans.PropertyChangeSupport;
import de.uks.se.castle.creators.RoomSet;
import de.uks.se.castle.creators.PersonSet;
import java.util.LinkedHashSet;
import org.sdmlib.serialization.json.JsonIdMap;
import de.uks.se.castle.creators.ItemSet;

public class Room implements PropertyChangeInterface
{

   
   //==========================================================================
   
   public Object get(String attrName)
   {
      if (PROPERTY_NR.equalsIgnoreCase(attrName))
      {
         return getNr();
      }

      if (PROPERTY_CASTLE.equalsIgnoreCase(attrName))
      {
         return getCastle();
      }

      if (PROPERTY_PERSONS.equalsIgnoreCase(attrName))
      {
         return getPersons();
      }

      if (PROPERTY_ITEMS.equalsIgnoreCase(attrName))
      {
         return getItems();
      }

      if (PROPERTY_PRINCESS.equalsIgnoreCase(attrName))
      {
         return getPrincess();
      }

      return null;
   }

   
   //==========================================================================
   
   public boolean set(String attrName, Object value)
   {
      if (PROPERTY_NR.equalsIgnoreCase(attrName))
      {
         setNr(Integer.parseInt(value.toString()));
         return true;
      }

      if (PROPERTY_CASTLE.equalsIgnoreCase(attrName))
      {
         setCastle((Castle) value);
         return true;
      }

      if (PROPERTY_PERSONS.equalsIgnoreCase(attrName))
      {
         addToPersons((Person) value);
         return true;
      }
      
      if ((PROPERTY_PERSONS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromPersons((Person) value);
         return true;
      }

      if (PROPERTY_ITEMS.equalsIgnoreCase(attrName))
      {
         addToItems((Item) value);
         return true;
      }
      
      if ((PROPERTY_ITEMS + JsonIdMap.REMOVE).equalsIgnoreCase(attrName))
      {
         removeFromItems((Item) value);
         return true;
      }

      if (PROPERTY_PRINCESS.equalsIgnoreCase(attrName))
      {
         setPrincess((Princess) value);
         return true;
      }

      return false;
   }

   
   //==========================================================================
   
   protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);
   
   public PropertyChangeSupport getPropertyChangeSupport()
   {
      return listeners;
   }

   
   //==========================================================================
   
   public void removeYou()
   {
      setCastle(null);
      removeAllFromPersons();
      removeAllFromItems();
      setPrincess(null);
      getPropertyChangeSupport().firePropertyChange("REMOVE_YOU", this, null);
   }

   
   //==========================================================================
   
   public static final String PROPERTY_NR = "nr";
   
   private int nr;

   public int getNr()
   {
      return this.nr;
   }
   
   public void setNr(int value)
   {
      if (this.nr != value)
      {
         int oldValue = this.nr;
         this.nr = value;
         getPropertyChangeSupport().firePropertyChange(PROPERTY_NR, oldValue, value);
      }
   }
   
   public Room withNr(int value)
   {
      setNr(value);
      return this;
   } 

   public String toString()
   {
      StringBuilder _ = new StringBuilder();
      
      _.append(" ").append(this.getNr());
      return _.substring(1);
   }


   
   public static final RoomSet EMPTY_SET = new RoomSet();

   
   /********************************************************************
    * <pre>
    *              many                       one
    * Room ----------------------------------- Castle
    *              rooms                   castle
    * </pre>
    */
   
   public static final String PROPERTY_CASTLE = "castle";
   
   private Castle castle = null;
   
   public Castle getCastle()
   {
      return this.castle;
   }
   
   public boolean setCastle(Castle value)
   {
      boolean changed = false;
      
      if (this.castle != value)
      {
         Castle oldValue = this.castle;
         
         if (this.castle != null)
         {
            this.castle = null;
            oldValue.withoutRooms(this);
         }
         
         this.castle = value;
         
         if (value != null)
         {
            value.withRooms(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_CASTLE, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Room withCastle(Castle value)
   {
      setCastle(value);
      return this;
   } 
   
   public Castle createCastle()
   {
      Castle value = new Castle();
      withCastle(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Room ----------------------------------- Person
    *              room                   persons
    * </pre>
    */
   
   public static final String PROPERTY_PERSONS = "persons";
   
   private PersonSet persons = null;
   
   public PersonSet getPersons()
   {
      if (this.persons == null)
      {
         return Person.EMPTY_SET;
      }
   
      return this.persons;
   }
   
   public boolean addToPersons(Person value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.persons == null)
         {
            this.persons = new PersonSet();
         }
         
         changed = this.persons.add (value);
         
         if (changed)
         {
            value.withRoom(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PERSONS, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromPersons(Person value)
   {
      boolean changed = false;
      
      if ((this.persons != null) && (value != null))
      {
         changed = this.persons.remove (value);
         
         if (changed)
         {
            value.setRoom(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_PERSONS, value, null);
         }
      }
         
      return changed;   
   }
   
   public Room withPersons(Person value)
   {
      addToPersons(value);
      return this;
   } 
   
   public Room withoutPersons(Person value)
   {
      removeFromPersons(value);
      return this;
   } 
   
   public void removeAllFromPersons()
   {
      LinkedHashSet<Person> tmpSet = new LinkedHashSet<Person>(this.getPersons());
   
      for (Person value : tmpSet)
      {
         this.removeFromPersons(value);
      }
   }
   
   public Person createPersons()
   {
      Person value = new Person();
      withPersons(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       many
    * Room ----------------------------------- Item
    *              room                   items
    * </pre>
    */
   
   public static final String PROPERTY_ITEMS = "items";
   
   private ItemSet items = null;
   
   public ItemSet getItems()
   {
      if (this.items == null)
      {
         return Item.EMPTY_SET;
      }
   
      return this.items;
   }
   
   public boolean addToItems(Item value)
   {
      boolean changed = false;
      
      if (value != null)
      {
         if (this.items == null)
         {
            this.items = new ItemSet();
         }
         
         changed = this.items.add (value);
         
         if (changed)
         {
            value.withRoom(this);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_ITEMS, null, value);
         }
      }
         
      return changed;   
   }
   
   public boolean removeFromItems(Item value)
   {
      boolean changed = false;
      
      if ((this.items != null) && (value != null))
      {
         changed = this.items.remove (value);
         
         if (changed)
         {
            value.setRoom(null);
            getPropertyChangeSupport().firePropertyChange(PROPERTY_ITEMS, value, null);
         }
      }
         
      return changed;   
   }
   
   public Room withItems(Item value)
   {
      addToItems(value);
      return this;
   } 
   
   public Room withoutItems(Item value)
   {
      removeFromItems(value);
      return this;
   } 
   
   public void removeAllFromItems()
   {
      LinkedHashSet<Item> tmpSet = new LinkedHashSet<Item>(this.getItems());
   
      for (Item value : tmpSet)
      {
         this.removeFromItems(value);
      }
   }
   
   public Item createItems()
   {
      Item value = new Item();
      withItems(value);
      return value;
   } 

   
   /********************************************************************
    * <pre>
    *              one                       one
    * Room ----------------------------------- Princess
    *              room                   princess
    * </pre>
    */
   
   public static final String PROPERTY_PRINCESS = "princess";
   
   private Princess princess = null;
   
   public Princess getPrincess()
   {
      return this.princess;
   }
   
   public boolean setPrincess(Princess value)
   {
      boolean changed = false;
      
      if (this.princess != value)
      {
         Princess oldValue = this.princess;
         
         if (this.princess != null)
         {
            this.princess = null;
            oldValue.setRoom(null);
         }
         
         this.princess = value;
         
         if (value != null)
         {
            value.withRoom(this);
         }
         
         getPropertyChangeSupport().firePropertyChange(PROPERTY_PRINCESS, oldValue, value);
         changed = true;
      }
      
      return changed;
   }
   
   public Room withPrincess(Princess value)
   {
      setPrincess(value);
      return this;
   } 
   
   public Princess createPrincess()
   {
      Princess value = new Princess();
      withPrincess(value);
      return value;
   } 
}

