/*
   Copyright (c) 2013 deeptought 
   
   Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
   and associated documentation files (the "Software"), to deal in the Software without restriction, 
   including without limitation the rights to use, copy, modify, merge, publish, distribute, 
   sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
   furnished to do so, subject to the following conditions: 
   
   The above copyright notice and this permission notice shall be included in all copies or 
   substantial portions of the Software. 
   
   The Software shall be used for Good, not Evil. 
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */
   
package de.uks.se.castle.creators;

import java.util.LinkedHashSet;
import de.uks.se.castle.Castle;
import org.sdmlib.models.modelsets.StringList;
import org.sdmlib.models.modelsets.intList;
import de.uks.se.castle.creators.RoomSet;
import de.uks.se.castle.Room;
import de.uks.se.castle.creators.PersonSet;
import de.uks.se.castle.Person;

public class CastleSet extends LinkedHashSet<Castle> implements org.sdmlib.models.modelsets.ModelSet
{


   public String toString()
   {
      StringList stringList = new StringList();
      
      for (Castle elem : this)
      {
         stringList.add(elem.toString());
      }
      
      return "(" + stringList.concat(", ") + ")";
   }


   public String getEntryType()
   {
      return "de.uks.se.castle.Castle";
   }


   public CastleSet with(Castle value)
   {
      this.add(value);
      return this;
   }
   
   public CastleSet without(Castle value)
   {
      this.remove(value);
      return this;
   }
   
   //==========================================================================
   
   public intList countAllItems()
   {
      intList result = new intList();
      for (Castle obj : this)
      {
         result.add(obj.countAllItems());
      }
      return result;
   }

   
   //==========================================================================
   
   public CastleSet princessSaved()
   {
      for (Castle obj : this)
      {
         obj.princessSaved();
      }
      return this;
   }

   public RoomSet getRooms()
   {
      RoomSet result = new RoomSet();
      
      for (Castle obj : this)
      {
         result.addAll(obj.getRooms());
      }
      
      return result;
   }

   public CastleSet withRooms(Room value)
   {
      for (Castle obj : this)
      {
         obj.withRooms(value);
      }
      
      return this;
   }

   public CastleSet withoutRooms(Room value)
   {
      for (Castle obj : this)
      {
         obj.withoutRooms(value);
      }
      
      return this;
   }

   public PersonSet getPersons()
   {
      PersonSet result = new PersonSet();
      
      for (Castle obj : this)
      {
         result.addAll(obj.getPersons());
      }
      
      return result;
   }

   public CastleSet withPersons(Person value)
   {
      for (Castle obj : this)
      {
         obj.withPersons(value);
      }
      
      return this;
   }

   public CastleSet withoutPersons(Person value)
   {
      for (Castle obj : this)
      {
         obj.withoutPersons(value);
      }
      
      return this;
   }

}

