package de.uks.se.castle.creators;

import org.sdmlib.models.pattern.PatternObject;
import de.uks.se.castle.Person;
import de.uks.se.castle.creators.PersonSet;
import org.sdmlib.models.pattern.AttributeConstraint;
import org.sdmlib.models.pattern.PatternLink;
import de.uks.se.castle.creators.CastlePO;
import org.sdmlib.models.pattern.LinkConstraint;
import de.uks.se.castle.creators.PersonPO;
import de.uks.se.castle.Castle;
import de.uks.se.castle.creators.RoomPO;
import de.uks.se.castle.Room;
import de.uks.se.castle.creators.ItemPO;
import de.uks.se.castle.Item;
import de.uks.se.castle.creators.ItemSet;
import de.uks.se.castle.creators.PrincessPO;
import de.uks.se.castle.Princess;

public class PersonPO extends PatternObject<PersonPO, Person>
{
   public PersonSet allMatches()
   {
      this.setDoAllMatches(true);
      
      PersonSet matches = new PersonSet();

      while (this.getPattern().getHasMatch())
      {
         matches.add((Person) this.getCurrentMatch());
         
         this.getPattern().findMatch();
      }
      
      return matches;
   }
   
   
   //==========================================================================
   
   public void collectItemsInCurrentRoom()
   {
      if (this.getPattern().getHasMatch())
      {
          ((Person) getCurrentMatch()).collectItemsInCurrentRoom();
      }
   }

   public PersonPO hasName(String value)
   {
      AttributeConstraint constr = (AttributeConstraint) new AttributeConstraint()
      .withAttrName(Person.PROPERTY_NAME)
      .withTgtValue(value)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier())
      .withPattern(this.getPattern());
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public String getName()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) getCurrentMatch()).getName();
      }
      return null;
   }
   
   public PersonPO withName(String value)
   {
      if (this.getPattern().getHasMatch())
      {
         ((Person) getCurrentMatch()).setName(value);
      }
      return this;
   }
   
   public PersonPO hasExperience(int value)
   {
      AttributeConstraint constr = (AttributeConstraint) new AttributeConstraint()
      .withAttrName(Person.PROPERTY_EXPERIENCE)
      .withTgtValue(value)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier())
      .withPattern(this.getPattern());
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public int getExperience()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) getCurrentMatch()).getExperience();
      }
      return 0;
   }
   
   public PersonPO withExperience(int value)
   {
      if (this.getPattern().getHasMatch())
      {
         ((Person) getCurrentMatch()).setExperience(value);
      }
      return this;
   }
   
   public CastlePO hasCastle()
   {
      CastlePO result = new CastlePO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Person.PROPERTY_CASTLE, result);
      
      return result;
   }

   public PersonPO hasCastle(CastlePO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Person.PROPERTY_CASTLE)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public Castle getCastle()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) this.getCurrentMatch()).getCastle();
      }
      return null;
   }

   public RoomPO hasRoom()
   {
      RoomPO result = new RoomPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Person.PROPERTY_ROOM, result);
      
      return result;
   }

   public PersonPO hasRoom(RoomPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Person.PROPERTY_ROOM)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public Room getRoom()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) this.getCurrentMatch()).getRoom();
      }
      return null;
   }

   public ItemPO hasItems()
   {
      ItemPO result = new ItemPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Person.PROPERTY_ITEMS, result);
      
      return result;
   }

   public PersonPO hasItems(ItemPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Person.PROPERTY_ITEMS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public ItemSet getItems()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) this.getCurrentMatch()).getItems();
      }
      return null;
   }

   public PrincessPO hasSavedPrincess()
   {
      PrincessPO result = new PrincessPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Person.PROPERTY_SAVEDPRINCESS, result);
      
      return result;
   }

   public PersonPO hasSavedPrincess(PrincessPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Person.PROPERTY_SAVEDPRINCESS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public Princess getSavedPrincess()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Person) this.getCurrentMatch()).getSavedPrincess();
      }
      return null;
   }

}

