package de.uks.se.castle.creators;

import org.sdmlib.models.pattern.PatternObject;
import de.uks.se.castle.Room;
import de.uks.se.castle.creators.RoomSet;
import org.sdmlib.models.pattern.AttributeConstraint;
import org.sdmlib.models.pattern.PatternLink;
import de.uks.se.castle.creators.CastlePO;
import org.sdmlib.models.pattern.LinkConstraint;
import de.uks.se.castle.creators.RoomPO;
import de.uks.se.castle.Castle;
import de.uks.se.castle.creators.PersonPO;
import de.uks.se.castle.Person;
import de.uks.se.castle.creators.PersonSet;
import de.uks.se.castle.creators.ItemPO;
import de.uks.se.castle.Item;
import de.uks.se.castle.creators.ItemSet;
import de.uks.se.castle.creators.PrincessPO;
import de.uks.se.castle.Princess;

public class RoomPO extends PatternObject<RoomPO, Room>
{
   public RoomSet allMatches()
   {
      this.setDoAllMatches(true);
      
      RoomSet matches = new RoomSet();

      while (this.getPattern().getHasMatch())
      {
         matches.add((Room) this.getCurrentMatch());
         
         this.getPattern().findMatch();
      }
      
      return matches;
   }
   
   public RoomPO hasNr(int value)
   {
      AttributeConstraint constr = (AttributeConstraint) new AttributeConstraint()
      .withAttrName(Room.PROPERTY_NR)
      .withTgtValue(value)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier())
      .withPattern(this.getPattern());
      
      this.getPattern().findMatch();
      
      return this;
   }
   
   public int getNr()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Room) getCurrentMatch()).getNr();
      }
      return 0;
   }
   
   public RoomPO withNr(int value)
   {
      if (this.getPattern().getHasMatch())
      {
         ((Room) getCurrentMatch()).setNr(value);
      }
      return this;
   }
   
   public CastlePO hasCastle()
   {
      CastlePO result = new CastlePO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Room.PROPERTY_CASTLE, result);
      
      return result;
   }

   public RoomPO hasCastle(CastlePO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Room.PROPERTY_CASTLE)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public Castle getCastle()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Room) this.getCurrentMatch()).getCastle();
      }
      return null;
   }

   public PersonPO hasPersons()
   {
      PersonPO result = new PersonPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Room.PROPERTY_PERSONS, result);
      
      return result;
   }

   public RoomPO hasPersons(PersonPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Room.PROPERTY_PERSONS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public PersonSet getPersons()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Room) this.getCurrentMatch()).getPersons();
      }
      return null;
   }

   public ItemPO hasItems()
   {
      ItemPO result = new ItemPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Room.PROPERTY_ITEMS, result);
      
      return result;
   }

   public RoomPO hasItems(ItemPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Room.PROPERTY_ITEMS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public ItemSet getItems()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Room) this.getCurrentMatch()).getItems();
      }
      return null;
   }

   public PrincessPO hasPrincess()
   {
      PrincessPO result = new PrincessPO();
      result.setModifier(this.getPattern().getModifier());
      
      super.hasLink(Room.PROPERTY_PRINCESS, result);
      
      return result;
   }

   public RoomPO hasPrincess(PrincessPO tgt)
   {
      LinkConstraint patternLink = (LinkConstraint) new LinkConstraint()
      .withTgt(tgt).withTgtRoleName(Room.PROPERTY_PRINCESS)
      .withSrc(this)
      .withModifier(this.getPattern().getModifier());
      
      this.getPattern().addToElements(patternLink);
      
      this.getPattern().findMatch();
      
      return this;
   }

   public Princess getPrincess()
   {
      if (this.getPattern().getHasMatch())
      {
         return ((Room) this.getCurrentMatch()).getPrincess();
      }
      return null;
   }

}

