package de.uks.se.pmws1213.sdmlib.tests;

import org.junit.Test;
import org.sdmlib.scenarios.Scenario;

import de.uks.se.castle.Castle;
import de.uks.se.castle.Item;
import de.uks.se.castle.Person;
import de.uks.se.castle.Princess;
import de.uks.se.castle.Room;

public class CastleTests
{
	/**
	 * Dieses Scenario soll absichern, dass die Modelltransformationen in der Methode Castle.countAllItems()
	 * smtliche Items in allen Rumen aufsummiert und die Anzahl zurck gibt.
	 */
	@Test
	public void countAllItemsTest()
	{
		Scenario scenario = new Scenario("genSrc");

		//startSituation
		Castle castle = new Castle();
		
		Room room = new Room()
		.withCastle(castle)
		.withItems(new Item().withDescription("Alter Schuh").withQuality(5))
		.withItems(new Item().withDescription("Grauer Umhang").withQuality(10))
		.withItems(new Item().withDescription("Bronzehelm").withQuality(19));

		Room room2 = new Room()
		.withCastle(castle)
		.withItems(new Item().withDescription("Fackel").withQuality(3));

		Room room3 = new Room()
		.withCastle(castle)
		.withItems(new Item().withDescription("Dolch").withQuality(7));

		Room room4 = new Room()
		.withCastle(castle)
		.withItems(new Item().withDescription("Lanze").withQuality(32))
		.withItems(new Item().withDescription("Brustpanzer").withQuality(23));


		//action
		int itemCount = castle.countAllItems();

		
		//resultSituation
		scenario.assertEquals("Wrong amount of items calculated", 7, itemCount);
	}

	/**
	 * In diesem Szenario soll der Prinz alle Gegenstnde im aktuellen Raum einsammeln.
	 * Die Gegenstnde mssen nicht explizit aus dem Raum entfernt werden - solange sie
	 * in die Itemliste des Prinzen eingetragen wurden und die Experience gestiegen ist, ist alles korrekt.
	 */
	@Test
	public void collectItemsInCurrentRoomTest()
	{
		Scenario scenario = new Scenario("genSrc");
		
		//startSituation
		Castle castle = new Castle();
		
		Room room = new Room()
		.withCastle(castle);

		Item diamondRing = new Item()
		.withRoom(room)
		.withDescription("Diamantring")
		.withQuality(128);
		
		Item rose = new Item()
		.withRoom(room)
		.withDescription("Rose")
		.withQuality(64);
		
		Item chocolateBox = new Item()
		.withRoom(room)
		.withDescription("Pralinenschachtel")
		.withQuality(8);

		Person prince = new Person()
		.withName("Prinz Eisenherz")
		.withRoom(room);
		
		
		//action
		prince.collectItemsInCurrentRoom();

		
		//resultSituation
		scenario.assertEquals("Wrong amount of collected Items.", 3, prince.getItems().size());
		scenario.assertEquals("Experience is not correct.", 200, prince.getExperience());
		scenario.assertTrue(diamondRing.getDescription() + " is not contained in " + prince.getName() + "s items.", prince.getItems().contains(diamondRing));
		scenario.assertTrue(rose.getDescription() + " is not contained in " + prince.getName() + "s items.", prince.getItems().contains(rose));
		scenario.assertTrue(chocolateBox.getDescription() + " is not contained in " + prince.getName() + "s items.", prince.getItems().contains(chocolateBox));
	}

	
	/**
	 * Hier soll das Game-Objekt erkennen, dass der Prinz im Raum mit der Prinzessin angekommen ist und
	 * einen Link zwischen beiden erstellen, um die Prinzessin als gerettet zu markieren.
	 */
	@Test
	public void princessSavedTest()
	{
		Scenario scenario = new Scenario("genSrc");
		
		//startSituation
		Castle castle = new Castle();
		
		Room room = new Room()
		.withCastle(castle);

		Room room2 = new Room()
		.withCastle(castle);

		Room room3 = new Room()
		.withCastle(castle);

		Room room4 = new Room()
		.withCastle(castle);
		
		Person prince = new Person()
		.withName("Prinz Eisenherz")
		.withRoom(room2);
		
		Princess princess = new Princess()
		.withName("Prinzessin Leia")
		.withRoom(room2);

		
		//action
		castle.princessSaved();
		
		
		//resultSituation
		scenario.assertTrue(princess.getName() + " has not been marked as saved.", princess == prince.getSavedPrincess());
	}
}
